# This file was auto-generated by Fern from our API Definition.

import typing

UpdateWeatherLayerApiV1DashboardWeatherLayersLayerIdPutRequestLayerType = typing.Union[
    typing.Literal["raster", "isoline", "particle"], typing.Any
]
