# This file was auto-generated by Fern from our API Definition.

import typing

CreateTimeseriesLiveApiV1DashboardDatasetsTimeseriesLivePostRequestDataFormat = typing.Union[
    typing.Literal["csv", "json"], typing.Any
]
